cbuffer MatrixBuffer : register(b0)
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
}

cbuffer ScreenBuffer : register(b1)
{
	float screenWidth;
	float3 padding;
}

struct InputType
{
	float4 pos : POSITION;
	float2 tex : TEXCOORD0;
	float3 norm : NORMAL;
};

struct OutputType
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 norm : NORMAL;
	float2 tex1 : TEXCOORD1;
	float2 tex2 : TEXCOORD2;
	float2 tex3 : TEXCOORD3;
	float2 tex4 : TEXCOORD4;
	float2 tex5 : TEXCOORD5;
	float2 tex6 : TEXCOORD6;
	float2 tex7 : TEXCOORD7;
	float2 tex8 : TEXCOORD8;
	float2 tex9 : TEXCOORD9;
};

OutputType main(InputType input)
{
	//pepare for pixel shader
	OutputType output;
	output.pos = input.pos;
	output.norm = input.norm;
	output.tex = input.tex;
	output.pos.w = 1;

	output.pos = mul(output.pos, worldMatrix);
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	//determine UVs of neighbours
	float texelSize = 1.0f / screenWidth;

	output.tex1 = input.tex + float2(texelSize*-4, 0.0f);
	output.tex2 = input.tex + float2(texelSize*-3, 0.0f);
	output.tex3 = input.tex + float2(texelSize*-2, 0.0f);
	output.tex4 = input.tex + float2(texelSize*-1, 0.0f);
	output.tex5 = input.tex + float2(texelSize*0, 0.0f);
	output.tex6 = input.tex + float2(texelSize*1, 0.0f);
	output.tex7 = input.tex + float2(texelSize*2, 0.0f);
	output.tex8 = input.tex + float2(texelSize*3, 0.0f);
	output.tex9 = input.tex + float2(texelSize*4, 0.0f);

	return output;
}