cbuffer MatrixBuffer : register(b0)
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
}

struct InputType
{
	float4 pos : POSITION;
	float2 tex : TEXCOORD0;
	float3 norm : NORMAL;
};

struct OutputType
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 norm : NORMAL;
	float4 shadowMapDepth : TEXCOORD6;
	float4 col : COLOR;
	matrix worldMatrix : TEXCOORD1;
	float4 worldPosition : TEXCOORD5;
};

[maxvertexcount(24)]
void main(point InputType input[1], inout TriangleStream<OutputType> triStream)
{
	OutputType output;
	output.tex = float2(0, 0);
	output.worldMatrix = worldMatrix;

	output.pos = mul(input[0].pos, worldMatrix);
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	//normals
	float3 FN = float3(0, 0, -1);
	float3 BN = float3(0, 0, 1);
	float3 LN = float3(-1, 0, 0);
	float3 RN = float3(1, 0, 0);
	float3 UN = float3(0, 1, 0);
	float3 DN = float3(0, -1, 0);

	//vertices
	float4 LTF = float4(-0.5, 0.5, -0.5,0);
	float4 LTB = float4(-0.5, 0.5, 0.5,0);
	float4 LBF = float4(-0.5, -0.5, -0.5,0);
	float4 LBB = float4(-0.5, -0.5, 0.5,0);
	float4 RTF = float4(0.5, 0.5, -0.5,0);
	float4 RTB = float4(0.5, 0.5, 0.5,0);
	float4 RBF = float4(0.5, -0.5, -0.5,0);
	float4 RBB = float4(0.5, -0.5, 0.5,0);
 
	if (input[0].norm.x + input[0].norm.y + input[0].norm.z == 1) //if not air
	{
		if (input[0].norm.z == 1) //if water
		{
			output.col = float4(0.4f, 1.7f, 1.7f, 0.5f);
		}
		else if (input[0].norm.y == 1) //if tree
		{
			int treeBranch;
			output.col = float4(0.5f, 0.25f, 0.3f, 1.0f);
			treeBranch = input[0].tex.x - 4096;
			if (treeBranch >= 0)
			{
				//Right Front
				LTF = float4(0.9, 0.1, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, -0.1, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, 0.1, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, -0.1, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 4096;
			}
			treeBranch -= 2048;
			if (treeBranch >= 0)
			{
				//Front
				LTF = float4(-0.1, 0.1, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, -0.1, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, 0.1, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, -0.1, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 2048;
			}
			treeBranch -= 1024;
			if (treeBranch >= 0)
			{
				//Left Front
				LTF = float4(-1.1, 0.1, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1.1, -0.1, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-0.9, 0.1, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-0.9, -0.1, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 1024;
			}
			treeBranch -= 512;
			if (treeBranch >= 0)
			{
				//Left
				LTF = float4(-1, 0.1, 0.1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1, -0.1, 0.1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-1, 0.1, -0.1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-1, -0.1, -0.1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 512;
			}
			treeBranch -= 256;
			if (treeBranch >= 0)
			{
				//Front Right Bottom
				LTF = float4(0.9, -0.9, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, -1.1, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, -0.9, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, -1.1, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 256;
			}
			treeBranch -= 128;
			if (treeBranch >= 0)
			{
				//Front Bottom
				LTF = float4(-0.1, -0.9, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, -1.1, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, -0.9, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, -1.1, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 128;
			}
			treeBranch -= 64;
			if (treeBranch >= 0)
			{
				//Left Front Bottom
				LTF = float4(-1.1, -0.9, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1.1, -1.1, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-0.9, -0.9, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-0.9, -1.1, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 64;
			}
			treeBranch -= 32;
			if (treeBranch >= 0)
			{
				//Right Bottom
				LTF = float4(0.9, -0.9, 0, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, -1.1, 0, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, -0.9, 0, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, -1.1, 0, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 32;
			}
			treeBranch -= 16;
			if (treeBranch >= 0)
			{
				//Bottom
				LTF = float4(-0.1, -0.9, 0, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, -1.1, 0, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, -0.9, 0, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, -1.1, 0, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 16;
			}
			treeBranch -= 8;
			if (treeBranch >= 0)
			{
				//Bottom Left
				LTF = float4(-1.1, -0.9, 0, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1.1, -1.1, 0, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-0.9, -0.9, 0, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-0.9, -1.1, 0, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 8;
			}
			treeBranch -= 4;
			if (treeBranch >= 0)
			{
				//Bottom Back Right
				LTF = float4(0.9, -0.9, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, -1.1, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, -0.9, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, -1.1, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 4;
			}
			treeBranch -= 2;
			if (treeBranch >= 0)
			{
				//Bottom Back
				LTF = float4(-0.1, -0.9, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, -1.1, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, -0.9, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, -1.1, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 2;
			}
			treeBranch -= 1;
			if (treeBranch >= 0)
			{
				//Bottom Back Left
				LTF = float4(-1.1, -0.9, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1.1, -1.1, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-0.9, -0.9, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-0.9, -1.1, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}

			treeBranch = input[0].tex.y - 4096;
			if (treeBranch >= 0)
			{
				//Back Left
				LTF = float4(-1, 0.1, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1, -0.1, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-1, 0.1, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-1, -0.1, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 4096;
			}
			treeBranch -= 2048;
			if (treeBranch >= 0)
			{
				//Back
				LTF = float4(-0.1, 0.1, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, -0.1, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, 0.1, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, -0.1, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 2048;
			}
			treeBranch -= 1024;
			if (treeBranch >= 0)
			{
				//Back Right
				LTF = float4(0.9, 0.1, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, -0.1, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, 0.1, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, -0.1, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 1024;
			}
			treeBranch -= 512;
			if (treeBranch >= 0)
			{
				//Right
				LTF = float4(0.9, 0.1, 0, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, -0.1, 0, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, 0.1, 0, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, -0.1, 0, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 512;
			}
			treeBranch -= 256;
			if (treeBranch >= 0)
			{
				//Back Left Top
				LTF = float4(-1.1, 1.1, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1.1, 0.9, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-0.9, 1.1, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-0.9, 0.9, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 256;
			}
			treeBranch -= 128;
			if (treeBranch >= 0)
			{
				//Back Top
				LTF = float4(-0.1, 1.1, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, 0.9, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, 1.1, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, 0.9, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 128;
			}
			treeBranch -= 64;
			if (treeBranch >= 0)
			{
				//Right Back Top
				LTF = float4(0.9, 1.1, 1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, 0.9, 1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, 1.1, 1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, 0.9, 1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 64;
			}
			treeBranch -= 32;
			if (treeBranch >= 0)
			{
				//Left Top
				LTF = float4(-1.1, 1.1, 0, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1.1, 0.9, 0, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-0.9, 1.1, 0, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-0.9, 0.9, 0, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 32;
			}
			treeBranch -= 16;
			if (treeBranch >= 0)
			{
				//Top
				LTF = float4(-0.1, -0.9, 0, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, -1.1, 0, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, -0.9, 0, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, -1.1, 0, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 16;
			}
			treeBranch -= 8;
			if (treeBranch >= 0)
			{
				//Top Right
				LTF = float4(0.9, 1.1, 0, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, 0.9, 0, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, 1.1, 0, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, 0.9, 0, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 8;
			}
			treeBranch -= 4;
			if (treeBranch >= 0)
			{
				//Top Front Left
				LTF = float4(-1.1, 1.1, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-1.1, 0.9, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(-0.9, 1.1, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(-0.9, 0.9, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 4;
			}
			treeBranch -= 2;
			if (treeBranch >= 0)
			{
				//Top Front
				LTF = float4(-0.1, 1.1, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(-0.1, 0.9, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(0.1, 1.1, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(0.1, 0.9, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			else
			{
				treeBranch += 2;
			}
			treeBranch -= 1;
			if (treeBranch >= 0)
			{
				//Top Front Right
				LTF = float4(0.9, 1.1, -1, 0);
				LTB = float4(-0.1, 0.1, 0, 0);
				LBF = float4(0.9, 0.9, -1, 0);
				LBB = float4(-0.1, -0.1, 0, 0);
				RTF = float4(1.1, 1.1, -1, 0);
				RTB = float4(0.1, 0.1, 0, 0);
				RBF = float4(1.1, 0.9, -1, 0);
				RBB = float4(0.1, -0.1, 0, 0);
			}
			
		}
		else if (input[0].norm.x == 1) //if ground
		{
			output.col = float4(0.2f, 0.6f, 0.0f, 1.0f);

			output.pos = mul(input[0].pos + float4(0,0.5,0,0), worldMatrix);
			output.pos = mul(output.pos, viewMatrix);
			output.pos = mul(output.pos, projectionMatrix);

			output.shadowMapDepth = output.pos;
		}
	}
	else //if air
	{
		output.col = float4(1.0f,1.0f,1.0f,0.0f);
	}

	//FRONT FACE
	output.norm = FN;
	output.norm = mul(output.norm, (float3x3)worldMatrix);
	output.norm = normalize(output.norm);

	output.pos = input[0].pos + LTF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;
	
	triStream.Append(output);

	output.pos = input[0].pos + LBF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RTF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RBF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);
	triStream.RestartStrip();

	//BACK FACE
	output.norm = BN;
	output.norm = mul(output.norm, (float3x3)worldMatrix);
	output.norm = normalize(output.norm);

	output.pos = input[0].pos + LTB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RTB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + LBB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RBB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);
	triStream.RestartStrip();

	//LEFT FACE
	output.norm = LN;
	output.norm = mul(output.norm, (float3x3)worldMatrix);
	output.norm = normalize(output.norm);

	output.pos = input[0].pos + LBB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + LBF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + LTB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + LTF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);
	triStream.RestartStrip();

	//RIGHT FACE
	output.norm = RN;
	output.norm = mul(output.norm, (float3x3)worldMatrix);
	output.norm = normalize(output.norm);

	output.pos = input[0].pos + RBB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RTB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RBF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RTF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);
	triStream.RestartStrip();

	//TOP FACE
	output.norm = UN;
	output.norm = mul(output.norm, (float3x3)worldMatrix);
	output.norm = normalize(output.norm);

	output.pos = input[0].pos + LTB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + LTF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RTB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RTF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);
	triStream.RestartStrip();

	//BOTTOM FACE
	output.norm = BN;
	output.norm = mul(output.norm, (float3x3)worldMatrix);
	output.norm = normalize(output.norm);

	output.pos = input[0].pos + LBB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RBB;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + LBF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);

	output.pos = input[0].pos + RBF;

	output.pos = mul(output.pos, worldMatrix);
	output.worldPosition = output.pos;
	output.pos = mul(output.pos, viewMatrix);
	output.pos = mul(output.pos, projectionMatrix);

	output.shadowMapDepth = output.pos;

	triStream.Append(output);
	triStream.RestartStrip();
}