cbuffer MatrixBuffer : register(b0)
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
};

struct InputType
{
	float4 position : POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
};

struct ConstantOutputType
{
	float edges[3] : SV_TESSFACTOR;
	float inside : SV_INSIDETESSFACTOR;
};

struct OutputType
{
	float4 position : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 colour : COLOR;
	matrix worldMatrix : TEXCOORD1;
	float4 worldPosition : TEXCOORD5;
};

[domain("tri")]
OutputType main(ConstantOutputType input, float3 uvwCoord : SV_DomainLocation, const OutputPatch<InputType, 3> patch)
{
	OutputType output;

	//set the normal
	output.normal = float3(0, 1, 0);

	//set a colour
	output.colour = 1.0f.xxxx;

	// Determine the position of the new vertex.
	float3 vertexPosition = uvwCoord.x * patch[0].position + uvwCoord.y * patch[1].position + uvwCoord.z * patch[2].position;

	//move it about a bit
	vertexPosition.y += sin(vertexPosition.x);
	vertexPosition.y += sin(vertexPosition.z);

	// Calculate the position of the new vertex against the world, view, and projection matrices.
	//also throw values to the pixel shader
	output.position = mul(float4(vertexPosition, 1.0f), worldMatrix);
	output.worldPosition = output.position;
	output.worldMatrix = worldMatrix;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	//make normals correct after you moved it
	output.normal.x = -cos(vertexPosition.x);
	output.normal.z = -cos(vertexPosition.z);

	output.normal = mul(output.normal, (float3x3) worldMatrix);

	output.tex = patch[0].tex;

	return output;
}