//how many steps to take while raymarching
#define STEPS 100
//scattering constant for phase function
#define SCATTERING 0.75

Texture2D shadowMap : register(t1);
SamplerState clampSamp : register(s1);

cbuffer LightCamera /*Action!*/ : register(b2)
{
	float3 cameraPosition;
	float lightSpecPower;
	matrix lightProjection;
	float4 lightAmbient;
	float4 lightDiffuse;
	float3 lightPosition;
	matrix lightView;
	float4 lightSpecColour;
};

struct InputType
{
	float4 position : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 colour : COLOR;
	matrix worldMatrix : TEXCOORD1;
	float4 worldPosition : TEXCOORD5;
};

float4 main(InputType input) : SV_TARGET
{
	float4 colour = lightAmbient;
	
	float4 specular = float4(0, 0, 0, 1);

	float3 lightDirection = normalize(lightPosition - input.worldPosition.xyz);

	//calculate vector to travel on
	float3 rayVector = input.worldPosition.xyz - cameraPosition;
	float rayScalar = length(rayVector);
	float3 rayDirection = rayVector / rayScalar;
	float3 step = rayDirection * STEPS;
	float4 currentPos = input.worldPosition;
	float4 fog = float4(0, 0, 0, 1);


		for (int i = 0; i < STEPS; ++i)
		{
			//get depth and coordinates
			float4 lightPerspective = mul(currentPos, lightView);
			lightPerspective = mul(lightPerspective, lightProjection);
			float lightDepth = lightPerspective.z / lightPerspective.w;
			float2 shadowTexCoord = lightPerspective.xy / lightPerspective.z;
			shadowTexCoord.x = (shadowTexCoord.x/2) + 0.5f;
			shadowTexCoord.y = (shadowTexCoord.y/2) + 0.5f;

			//check if it is in range
			if ((saturate(shadowTexCoord.x) == shadowTexCoord.x) && (saturate(shadowTexCoord.y) == shadowTexCoord.y))
			{
				//sample the shadow map
				float shadowMapValue = shadowMap.Sample(clampSamp, shadowTexCoord).r;
				//compare to depth
				if (shadowMapValue < lightDepth)
				{
					//scatter the light as it travels
					float additionalFog = 1 - SCATTERING*SCATTERING;
					additionalFog /= (4 * 3.14159f*pow(1 + SCATTERING*SCATTERING - (2 * SCATTERING) * (dot(rayDirection, lightDirection)), 1.5f));
					fog += float4(additionalFog.xxx, 0) * lightDiffuse;

					//only do this for the vertex itself
					if (i == 0)
					{
						//regular lighting
						float intensity = saturate(dot(input.normal, lightDirection));
						if (intensity > 0)
						{
							colour += lightDiffuse*intensity;
						}
					}
				}


				currentPos += float4(step, 0);
			}
	}
	//average out the fog
	fog /= STEPS;
	colour = saturate(colour + fog);

	return colour;
}