cbuffer MatrixBuffer : register(b0)
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
};

struct InputType
{
	float4 position : POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 rotationID : TEXCOORD1;
};

struct OutputType
{
	float4 position : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 colour : COLOR;
	matrix worldMatrix : TEXCOORD1;
	float4 worldPosition : TEXCOORD5;
};

[maxvertexcount(24)]
void main(point InputType input[1], inout TriangleStream<OutputType> triStream)
{
	OutputType output;
	
	float4 modifier;

	//diferent colour for each face
	output.colour = float4(1.0f, 1.0f, 1.0f, 1.0f); //white

	
	
	//move to a corner
	modifier = float4(-0.5f, 0.5f, -0.5f, 0.0f);
	output.normal = float3(0, 0, -1);
	output.tex = float2(0.0f, 0.0f);
	
	//rotate the cubes like the vertices have been
	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f,	0.0f,					0.0f,
														0.0f,	cos(input[0].rotationID.x),	0.0f-sin(input[0].rotationID.x),
														0.0f,	sin(input[0].rotationID.x),	cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f,	0.0f,					0.0f,
													0.0f,	cos(input[0].rotationID.x),	0.0f-sin(input[0].rotationID.x),
													0.0f,	sin(input[0].rotationID.x),	cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y),	0.0f,	sin(input[0].rotationID.y),
														0.0f,					1.0f,	0.0f,
														0.0f-sin(input[0].rotationID.y),	0.0f,	cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y),	0.0f,	sin(input[0].rotationID.y),
													0.0f,					1.0f,	0.0f,
													0.0f-sin(input[0].rotationID.y),	0.0f,	cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z),	0.0f-sin(input[0].rotationID.z),	0.0f,
														sin(input[0].rotationID.z),	cos(input[0].rotationID.z),	0.0f,
														0.0f,					0.0f,					1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z),	0.0f-sin(input[0].rotationID.z),	0.0f,
													sin(input[0].rotationID.z),	cos(input[0].rotationID.z),	0.0f,
													0.0f,					0.0f,					1.0f));
	}

	output.position = input[0].position + modifier;
	
	//project it to the screen
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	//repeat for next corner
	triStream.Append(output);

	modifier = float4(-0.5f, -0.5f, -0.5f, 0.0f);
	output.normal = float3(0, 0, -1);
	output.tex = float2(0.0f, 1.0f);

	

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, 0.5f, -0.5f, 0.0f);
	output.normal = float3(0, 0, -1);
	output.tex = float2(1.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, -0.5f, -0.5f, 0.0f);
	output.normal = float3(0, 0, -1);
	output.tex = float2(1.0f, 1.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);
	triStream.Append(output);

	//repeat for next face
	triStream.RestartStrip();

	output.colour = float4(0.75f, 0.5f, 0.0f, 1.0f); //orange

	modifier = float4(0.5f, -0.5f, 0.5f, 0.0f);
	output.normal = float3(1, 0, 0);
	output.tex = float2(1.0f, 1.0f);

	

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, 0.5f, 0.5f, 0.0f);
	output.normal = float3(1, 0, 0);
	output.tex = float2(1.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, -0.5f, -0.5f, 0.0f);
	output.normal = float3(1, 0, 0);
	output.tex = float2(0.0f, 1.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	modifier = float4(0.5f, 0.5f, -0.5f, 0.0f);
	output.normal = float3(1, 0, 0);
	output.tex = float2(0.0f, 0.0f);

	

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);
	triStream.Append(output);
	triStream.RestartStrip();

	output.colour = float4(0.0f, 1.0f, 0.0f, 1.0f); //green

	modifier = float4(-0.5f, 0.5f, 0.5f, 0.0f);
	output.normal = float3(0, 1, 0);
	output.tex = float2(1.0f, 1.0f);

	

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(-0.5f, 0.5f, -0.5f, 0.0f);
	output.normal = float3(0, 1, 0);
	output.tex = float2(1.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	modifier = float4(0.5f, 0.5f, 0.5f, 0.0f);
	output.normal = float3(0, 1, 0);
	output.tex = float2(0.0f, 1.0f);

	

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, 0.5f, -0.5f, 0.0f);
	output.normal = float3(0, 1, 0);
	output.tex = float2(0.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);
	triStream.Append(output);
	triStream.RestartStrip();

	output.colour = float4(1.0f, 1.0f, 0.0f, 1.0f); // yellow

	

	

	modifier = float4(-0.5f, 0.5f, 0.5f, 0.0f);
	output.normal = float3(0, 0, 1);
	output.tex = float2(1.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, 0.5f, 0.5f, 0.0f);
	output.normal = float3(0, 0, 1);
	output.tex = float2(0.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(-0.5f, -0.5f, 0.5f, 0.0f);
	output.normal = float3(0, 0, 1);
	output.tex = float2(1.0f, 1.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, -0.5f, 0.5f, 0.0f);
	output.normal = float3(0, 0, 1);
	output.tex = float2(0.0f, 1.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);
	triStream.Append(output);
	triStream.RestartStrip();

	output.colour = float4(0.0f, 0.0f, 1.0f, 1.0f); //blue

	modifier = float4(-0.5f, -0.5f, 0.5f, 0.0f);
	output.normal = float3(0, -1, 0);
	output.tex = float2(1.0f, 1.0f);

	

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	modifier = float4(0.5f, -0.5f, 0.5f, 0.0f);
	output.normal = float3(0, -1, 0);
	output.tex = float2(0.0f, 1.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(-0.5f, -0.5f, -0.5f, 0.0f);
	output.normal = float3(0, -1, 0);
	output.tex = float2(1.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(0.5f, -0.5f, -0.5f, 0.0f);
	output.normal = float3(0, -1, 0);
	output.tex = float2(0.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);
	triStream.Append(output);
	triStream.RestartStrip();

	output.colour = float4(1.0f, 0.0f, 0.0f, 1.0f); //red

	modifier = float4(-0.5f, -0.5f, 0.5f, 0.0f);
	output.normal = float3(-1, 0, 0);
	output.tex = float2(1.0f, 1.0f);

	

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(-0.5f, -0.5f, -0.5f, 0.0f);
	output.normal = float3(-1, 0, 0);
	output.tex = float2(1.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(-0.5f, 0.5f, 0.5f, 0.0f);
	output.normal = float3(-1, 0, 0);
	output.tex = float2(0.0f, 1.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);

	triStream.Append(output);

	

	modifier = float4(-0.5f, 0.5f, -0.5f, 0.0f);
	output.normal = float3(-1, 0, 0);
	output.tex = float2(0.0f, 0.0f);

	if (input[0].rotationID.w == 1.0f || input[0].rotationID.w == 3 || input[0].rotationID.w == 5 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
		output.normal = mul(output.normal, float3x3(1.0f, 0.0f, 0.0f,
			0.0f, cos(input[0].rotationID.x), 0.0f - sin(input[0].rotationID.x),
			0.0f, sin(input[0].rotationID.x), cos(input[0].rotationID.x)));
	}
	if (input[0].rotationID.w == 2 || input[0].rotationID.w == 3 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.y), 0.0f, sin(input[0].rotationID.y),
			0.0f, 1.0f, 0.0f,
			0.0f - sin(input[0].rotationID.y), 0.0f, cos(input[0].rotationID.y)));
	}
	if (input[0].rotationID.w == 4 || input[0].rotationID.w == 5 || input[0].rotationID.w == 6 || input[0].rotationID.w == 7)
	{
		modifier.xyz = mul(modifier.xyz, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
		output.normal = mul(output.normal, float3x3(cos(input[0].rotationID.z), 0.0f - sin(input[0].rotationID.z), 0.0f,
			sin(input[0].rotationID.z), cos(input[0].rotationID.z), 0.0f,
			0.0f, 0.0f, 1.0f));
	}
	output.position = input[0].position + modifier;
	output.position = mul(output.position, worldMatrix);
	output.worldMatrix = worldMatrix;
	output.worldPosition = output.position;
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	output.normal = mul(output.normal, (float3x3)worldMatrix);
	output.normal = normalize(output.normal);
	triStream.Append(output);
	triStream.RestartStrip();
}