cbuffer RotationBuffer : register(b1)
{
	float4 rotation;
};

struct InputType
{
	float4 position : POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
};

struct OutputType
{
	float4 position : POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 rotationID : TEXCOORD1;
};

OutputType main(InputType input)
{
	OutputType output;

	// Change the position vector to be 4 units for proper matrix calculations.
	input.position.w = 1.0f;

	//change the rotation to radians for sine and cosine functions
	output.rotationID.x = radians(rotation.x);
	output.rotationID.y = radians(rotation.y);
	output.rotationID.z = radians(rotation.z);
	//w keeps track of what axes the points have been rotated on
	output.rotationID.w = 0;

	//rotate points around the any axes they do not sit directly on, first x
	if (input.position.x != 0)
	{
		input.position.xyz = mul(input.position.xyz, float3x3(	1,	0,					0,
																0,	cos(output.rotationID.x),	0-sin(output.rotationID.x),
																0,	sin(output.rotationID.x),	cos(output.rotationID.x)));
		output.rotationID.w += 1;
	}
	//then any points that are not directly on y after being rotated around x where applicable
	if (input.position.y != 0)
	{
		input.position.xyz = mul(input.position.xyz, float3x3(	cos(output.rotationID.y),	0,	sin(output.rotationID.y),
																0,					1,	0,
																0-sin(output.rotationID.y),	0,	cos(output.rotationID.y)));
		output.rotationID.w += 2;
	}
	//and again with z
	if (input.position.z != 0)
	{
		input.position.xyz = mul(input.position.xyz, float3x3(	cos(output.rotationID.z),	0-sin(output.rotationID.z),	0,
																sin(output.rotationID.z),	cos(output.rotationID.z),	0,
																0,					0,					1));
		output.rotationID.w += 4;
	}
	
	//Store position
	output.position = input.position;

	// Store the texture coordinates
	output.tex = input.tex;

	// Store normals
	output.normal = input.normal;

	return output;
}